#ifndef LIBHTDIG_API_H
#define LIBHTDIG_API_H

//----------------------------------------------------------------
//
// header function for htdig & htmerge callable API
//
// 1/25/2001 created
//
// nealr@rightnow.com
//
//----------------------------------------------------------------


#define MAX_FILENAME_PATH_L  1024

typedef struct htdig_parameters_struct {

  char configFile[MAX_FILENAME_PATH_L];
  char credentials[MAX_FILENAME_PATH_L];
  char max_hops[10];    //9 digit limit
  char minimalFile[MAX_FILENAME_PATH_L];
  char URL[MAX_FILENAME_PATH_L];

  //debugging & logfile
  char logFile[MAX_FILENAME_PATH_L];   //location of log file
  int debug;            //0, 1 ,2, 3, 4, 5
  
  //booelan values
  int initial;
  int create_text_database;
  int report_statistics;
  int alt_work_area;

} htdig_parameters_struct;

/*  DOCUMENTATION for htdig_parameters_struct

    DEBUGGING PARAMETERS

    int debug 
        Verbose mode.  This increases the verbosity of the
         program.  Using more than 2 is probably only useful
         for debugging purposes.  The default verbose mode
         gives a nice progress report while digging.

    char logFile
         File to stream debugging & error messages to!
    
    BOOLEAN PARAMETERS

    int initial
         Initial.  Do not use any old databases.  This is
        accomplished by first erasing the databases

    int create_text_database
         Create an ASCII version of the document database.
        This database is easy to parse with other programs so
         that information can be extracted from it.

    int report_statistics
         Report statistics after completion.

    int alt_work_area
         Use alternate work files.
        Tells htdig to append .work to database files, causing
        a second copy of the database to be built.  This allows
        the original files to be used by htsearch during the
        indexing run.
 
        
    STRING PARAMETERS

    char configFile
         configfile
         Use the specified configuration file instead of the
         default.

    char credentials
        username:password
        Tells htdig to send the supplied username and
        password with each HTTP request.  The credentials
        will be encoded using the 'Basic' authentication scheme.
        There *HAS* to be a colon (:) between the username
        and password.


    char maxhops    //9 digit limit
         hopcount
         Limit the stored documents to those which are at
         most hopcount links away from the start URL.

    char minimalFile

    char URL
         'command-line' URLs from stdin
         fetches & indexes these URLs

*/

int htdig_index_open(htdig_parameters_struct *);
int htdig_index_doc(void);
int htdig_index_urls(void);
int htdig_index_close(void);


typedef struct htmerge_parameters_struct {

  char configFile[MAX_FILENAME_PATH_L];
  char merge_configFile[MAX_FILENAME_PATH_L];

  //debugging & logfile
  char logFile[MAX_FILENAME_PATH_L];   //location of log file
  int debug;            //0, 1 ,2, 3, 4, 5
  
  //booelan values
  int alt_work_area;

} htmerge_parameters_struct;

/*  DOCUMENTATION for  htmerge_parameters_struct
 
    DEBUGGING PARAMETERS

    int debug 
        Verbose mode.  This increases the verbosity of the
        program.  Using more than 2 is probably only useful
        for debugging purposes.  The default verbose mode
        gives a progress on what it is doing and where it is.

    char logFile
         File to stream debugging & error messages to!

    
    BOOLEAN PARAMETERS

    int alt_work_area
        Use alternate work files.
        Tells htmerge to append .work to database files causing
        a second copy of the database to be built.  This allows
        original files to be used by htsearch during the indexing run.


    STRING PARAMETERS
        
    char configFile
        configfile
        Use the specified configuration file instead of the default.

    char merge_configFile
        merge_configfile
        Merge the databases specified into the databases specified
        by -c or the default.


*/

int htmerge_index_merge(htmerge_parameters_struct *);


#endif /* LIBHTDIG_API_H */

