//
// htdig.cc
// 
// htdig: Indexes the web sites specified in the config file
//        generating several databases to be used by htmerge
//
// Part of the ht://Dig package   <http://www.htdig.org/>
// Copyright (c) 1995-2001 The ht://Dig Group
// For copyright details, see the file COPYING in your distribution
// or the GNU Public License version 2 or later
// <http://www.gnu.org/copyleft/gpl.html>
//
// $Id: htdig.cc,v 1.26.2.23 2001/12/17 05:47:46 angusgb Exp $
//
//  nealr@rightnow.com  made htdig-exe into callable function.

extern "C" {
#include "libhtdig_api.h"
}


#ifdef HAVE_CONFIG_H
#include "htconfig.h"
#endif /* HAVE_CONFIG_H */

#include "Document.h"
#include "Retriever.h"
#include "StringList.h"
#include "htdig.h"
#include "defaults.h"
#include "HtURLCodec.h"
#include "WordContext.h"
#include "HtDateTime.h"
#include "HtURLRewriter.h"

////////////////////////////
// For cookie jar
////////////////////////////
#include "HtCookieJar.h"
#include "HtCookieMemJar.h"
#include "HtHTTP.h"
////////////////////////////

// If we have this, we probably want it.
//#ifdef HAVE_GETOPT_H
//#include <getopt.h>
//#endif

#include <iostream.h>


//Global Variables for Library

int debug = 0;
HtRegexList limits;
HtRegexList limitsn;
String configFile = DEFAULT_CONFIG_FILE;
FILE *urls_seen = NULL;
FILE *images_seen = NULL;
DocumentDB docs;

//
// Global variables for this file
//
static int report_statistics = 0;
static String minimalFile = 0;
static HtDateTime StartTime;
static HtDateTime EndTime;

static char *max_hops = NULL;
static String credentials;
static HtCookieJar *_cookie_jar = NULL;
static HtConfiguration * config = NULL;

static int create_text_database = 0;
static int alt_work_area = 0;
static int initial = 0;

//new.  URLs from 'command-line'
#define URL_SEPCHARS    " ,"
static char *URL = NULL;


static FILE *errorlog_fp = NULL;

//internal functions
static void reportError_htdig(char *msg);


int htdig_index_open(htdig_parameters_struct * htdig_parms)
{

    //load 'comand-line' parameters
    
    if (htdig_parms->configFile[0] != 0)
        configFile = htdig_parms->configFile;
    
    if (htdig_parms->URL[0] != 0)
        URL = strdup(htdig_parms->URL);
    
    debug = htdig_parms->debug;
    if(debug != 0)
    {
        errorlog_fp = fopen(htdig_parms->logFile , "a+");

        if(errorlog_fp == NULL)
        {
            fprintf(stderr, "htdig: Error opening file [%s]. Error:[%d], %s\n", 
                    htdig_parms->logFile, errno, strerror(errno));
        }
    }
    
    initial = htdig_parms->initial;
    create_text_database  =  htdig_parms->create_text_database;
    max_hops  =  strdup(htdig_parms->max_hops);
    report_statistics  =  htdig_parms->report_statistics;
    credentials = htdig_parms->credentials;
    alt_work_area  =  htdig_parms->alt_work_area;
    minimalFile = htdig_parms->minimalFile;

	// Cookie jar dynamic creation.
	_cookie_jar = new HtCookieMemJar ();	// new cookie jar
	if (_cookie_jar)
		HtHTTP::SetCookieJar (_cookie_jar);

	//
	// First set all the defaults and then read the specified config
	// file to override the defaults.
	//
    config = HtConfiguration::config ();
	config->Defaults (&defaults[0]);
	if (access ((char *) configFile, R_OK) < 0)
	{
		reportError_htdig (form ("Unable to find configuration file '%s'",
					    configFile.get ()));
	}
	config->Read (configFile);

	if (config->Find ("locale").empty () && debug > 0)
		cout << "Warning: unknown locale!\n";

	if (max_hops)
	{
		config->Add ("max_hop_count", max_hops);
	}

	// Set up credentials for this run
	if (credentials.length ())
		config->Add ("authorization", credentials);

	//
	// Check url_part_aliases and common_url_parts for
	// errors.
	String url_part_errors = HtURLCodec::instance ()->ErrMsg ();

	if (url_part_errors.length () != 0)
		reportError_htdig (form
				   ("Invalid url_part_aliases or common_url_parts: %s",
				    url_part_errors.get ()));

	//
	// Check url_rewrite_rules for errors.
	String url_rewrite_rules = HtURLRewriter::instance ()->ErrMsg ();

	if (url_rewrite_rules.length () != 0)
		reportError_htdig (form ("Invalid url_rewrite_rules: %s",
					    url_rewrite_rules.get ()));

	//
	// If indicated, change the database file names to have the .work
	// extension
	//
	if (alt_work_area != 0)
	{
		String configValue = config->Find ("doc_db");

		if (configValue.length () != 0)
		{
			configValue << ".work";
			config->Add ("doc_db", configValue);
		}

		configValue = config->Find ("word_db");
		if (configValue.length () != 0)
		{
			configValue << ".work";
			config->Add ("word_db", configValue);
		}

		configValue = config->Find ("doc_index");
		if (configValue.length () != 0)
		{
			configValue << ".work";
			config->Add ("doc_index", configValue);
		}

		configValue = config->Find ("doc_excerpt");
		if (configValue.length () != 0)
		{
			configValue << ".work";
			config->Add ("doc_excerpt", configValue);
		}

		configValue = config->Find ("md5_db");
		if (configValue.length () != 0)
		{
			configValue << ".work";
			config->Add ("md5_db", configValue);
		}
	}

	//
	// If needed, we will create a list of every URL we come across.
	//
	if (config->Boolean ("create_url_list"))
	{
		const String filename = config->Find ("url_list");
		urls_seen = fopen (filename, initial ? "w" : "a");
		if (urls_seen == 0)
		{
			reportError_htdig (form ("Unable to create URL file '%s'",
						    filename.get ()));
		}
	}

	//
	// If needed, we will create a list of every image we come across.
	//
	if (config->Boolean ("create_image_list"))
	{
		const String filename = config->Find ("image_list");
		images_seen = fopen (filename, initial ? "w" : "a");
		if (images_seen == 0)
		{
			reportError_htdig (form ("Unable to create images file '%s'",
						    filename.get ()));
		}
	}

	//
	// Set up the limits list
	//
	StringList l (config->Find ("limit_urls_to"), " \t");
	limits.setEscaped (l, config->Boolean ("case_sensitive"));
	l.Destroy ();

	l.Create (config->Find ("limit_normalized"), " \t");
	limitsn.setEscaped (l, config->Boolean ("case_sensitive"));
	l.Destroy ();

	//
	// Open the document database
	//
	const String filename = config->Find ("doc_db");
	if (initial)
		unlink (filename);

	const String index_filename = config->Find ("doc_index");
	if (initial)
		unlink (index_filename);

	const String head_filename = config->Find ("doc_excerpt");
	if (initial)
		unlink (head_filename);

	if (docs.Open (filename, index_filename, head_filename) < 0)
	{
		reportError_htdig (form ("Unable to open/create document database '%s'",
					    filename.get ()));
	}

	const String word_filename = config->Find ("word_db");
	if (initial)
		unlink (word_filename);

	// Initialize htword
	WordContext::Initialize (*config);

    return(1);

}

int htdig_index_doc(void)
{
 return(1);
}

int htdig_index_urls(void)
{

    char * temp_URL_list = NULL;
    char * temp_url = NULL;

	// Create the Retriever object which we will use to parse all the
	// HTML files.
	// In case this is just an update dig, we will add all existing
	// URLs?
	//
	Retriever retriever (Retriever_logUrl);
	if (minimalFile.length () == 0)
	{
		List *list = docs.URLs ();
		retriever.Initial (*list);
		delete list;

		// Add start_url to the initial list of the retriever.
		// Don't check a URL twice!
		// Beware order is important, if this bugs you could change 
		// previous line retriever.Initial(*list, 0) to Initial(*list,1)
		retriever.Initial (config->Find ("start_url"), 1);
	}

	// Handle list of URLs given on 'command-line'
	if (URL != NULL)
	{
		String str;
        temp_URL_list = strdup(URL);
        temp_url = strtok(temp_URL_list, URL_SEPCHARS);
		while (temp_url != NULL)
		{
			str = temp_url;
			str.chop ("\r\n");
			if (str.length () > 0)
				retriever.Initial (str, 1);

            temp_url = strtok(NULL, URL_SEPCHARS);
		}
        free(temp_URL_list);
	}
	else if (minimalFile.length () != 0)
	{
		FILE *input = fopen (minimalFile.get (), "r");
		char buffer[1000];

		if (input)
		{
			while (fgets (buffer, sizeof (buffer), input))
			{
				String str (buffer);
				str.chop ("\r\n\t ");
				if (str.length () > 0)
					retriever.Initial (str, 1);
			}
			fclose (input);
		}
	}

	//
	// Go do it!
	//
	retriever.Start ();

	//
	// All done with parsing.
	//

	//
	// If the user so wants, create a text version of the document database.
	//

	if (create_text_database)
	{
		const String doc_list = config->Find ("doc_list");
		if (initial)
			unlink (doc_list);
		docs.DumpDB (doc_list);
		const String word_dump = config->Find ("word_dump");
		if (initial)
			unlink (word_dump);
		HtWordList words (*config);
		if (words.Open (config->Find ("word_db"), O_RDONLY) == OK)
		{
			words.Dump (word_dump);
		}
	}

	//
	// Cleanup
	//
	if (urls_seen)
		fclose (urls_seen);
	if (images_seen)
		fclose (images_seen);

	//
	// If needed, report some statistics
	//
	if (report_statistics)
	{
		retriever.ReportStatistics ("htdig");
	}

     return(1);
}


int htdig_index_close(void)
{

	if (_cookie_jar)
		delete _cookie_jar;

    if (max_hops != NULL)
        free(max_hops);

    if (URL != NULL)
        free(URL);

    //call destructors here
    docs.~DocumentDB();
    config->~HtConfiguration();

    if(errorlog_fp != NULL)
        fclose(errorlog_fp);

    return(1);
}


//
// Report an error and die
//
void
static reportError_htdig(char *msg)
{
    time_t now = time(NULL);
    
    if(errorlog_fp != NULL)
    	fprintf(errorlog_fp, "%s  [HTDIG] %s\n", ctime(&now), msg);

    
	//exit (1); 
}

//
// Report an error and die
//
void reportError(char *msg)
{
    cout << "htdig: " << msg << "\n\n";
    exit(1);
}

